@@ -25,6 +25,6 @@
 
 defined('MOODLE_INTERNAL') || die();
 
-$plugin->version   = 2011120100;        // The current plugin version (Date: YYYYMMDDXX)
-$plugin->requires  = 2011120100;        // Requires this Moodle version
-$plugin->component = 'format_weeks';    // Full name of the plugin (used for diagnostics)
+$plugin->version   = 2014051200;        // The current plugin version (Date: YYYYMMDDXX).
+$plugin->requires  = 2014050800;        // Requires this Moodle version.
+$plugin->component = 'format_weeks';    // Full name of the plugin (used for diagnostics).
