@@ -1,12 +1,26 @@
 <?php
+// This file is part of Moodle - http://moodle.org/
+//
+// Moodle is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// Moodle is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
+
+
 /**
  * The main group management user interface.
  *
- * @copyright &copy; 2006 The Open University
- * @author N.D.Freear AT open.ac.uk
- * @author J.White AT open.ac.uk
- * @license http://www.gnu.org/copyleft/gpl.html GNU Public License
- * @package groups
+ * @copyright 2006 The Open University, N.D.Freear AT open.ac.uk, J.White AT open.ac.uk
+ * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
+ * @package   core_group
  */
 require_once('../config.php');
 require_once('lib.php');
@@ -42,13 +56,10 @@ $PAGE->set_url($url);
 // Make sure that the user has permissions to manage groups.
 require_login($course);
 
-$PAGE->requires->yui2_lib('connection');
-$PAGE->requires->js('/group/clientlib.js');
+$context = context_course::instance($course->id);
+require_capability('moodle/course:managegroups', $context);
 
-$context = get_context_instance(CONTEXT_COURSE, $course->id);
-if (!has_capability('moodle/course:managegroups', $context)) {
-    redirect('/course/view.php', array('id'=>$course->id)); // Not allowed to manage all groups
-}
+$PAGE->requires->js('/group/clientlib.js');
 
 // Check for multiple/no group errors
 if (!$singlegroup) {
@@ -67,7 +78,7 @@ switch ($action) {
 
     case 'ajax_getmembersingroup':
         $roles = array();
-        if ($groupmemberroles = groups_get_members_by_role($groupids[0], $courseid, 'u.id,u.firstname,u.lastname')) {
+        if ($groupmemberroles = groups_get_members_by_role($groupids[0], $courseid, 'u.id, ' . get_all_user_name_fields(true, 'u'))) {
             foreach($groupmemberroles as $roleid=>$roledata) {
                 $shortroledata = new stdClass();
                 $shortroledata->name = $roledata->name;
@@ -141,19 +152,11 @@ $currenttab = 'groups';
 require('tabs.php');
 
 $disabled = 'disabled="disabled"';
-if (ajaxenabled()) {
-    // Some buttons are enabled if single group selected
-    $showaddmembersform_disabled = $singlegroup ? '' : $disabled;
-    $showeditgroupsettingsform_disabled = $singlegroup ? '' : $disabled;
-    $deletegroup_disabled = count($groupids)>0 ? '' : $disabled;
-} else {
-    // Do not disable buttons. The buttons work based on the selected group,
-    // which you can change without reloading the page, so it is not appropriate
-    // to disable them if no group is selected.
-    $showaddmembersform_disabled = '';
-    $showeditgroupsettingsform_disabled = '';
-    $deletegroup_disabled = '';
-}
+
+// Some buttons are enabled if single group selected.
+$showaddmembersform_disabled = $singlegroup ? '' : $disabled;
+$showeditgroupsettingsform_disabled = $singlegroup ? '' : $disabled;
+$deletegroup_disabled = count($groupids) > 0 ? '' : $disabled;
 
 echo $OUTPUT->heading(format_string($course->shortname, true, array('context' => $context)) .' '.$strgroups, 3);
 echo '<form id="groupeditform" action="index.php" method="post">'."\n";
@@ -167,17 +170,14 @@ echo '<tr>'."\n";
 echo "<td>\n";
 echo '<p><label for="groups"><span id="groupslabel">'.get_string('groups').':</span><span id="thegrouping">&nbsp;</span></label></p>'."\n";
 
-if (ajaxenabled()) { // TODO: move this to JS init!
-    $onchange = 'M.core_group.membersCombo.refreshMembers();';
-} else {
-    $onchange = '';
-}
+$onchange = 'M.core_group.membersCombo.refreshMembers();';
 
 echo '<select name="groups[]" multiple="multiple" id="groups" size="15" class="select" onchange="'.$onchange.'"'."\n";
 echo ' onclick="window.status=this.selectedIndex==-1 ? \'\' : this.options[this.selectedIndex].title;" onmouseout="window.status=\'\';">'."\n";
 
 $groups = groups_get_all_groups($courseid);
 $selectedname = '&nbsp;';
+$preventgroupremoval = array();
 
 if ($groups) {
     // Print out the HTML
@@ -192,6 +192,9 @@ if ($groups) {
                 $selectedname = $groupname;
             }
         }
+        if (!empty($group->idnumber) && !has_capability('moodle/course:changeidnumber', $context)) {
+            $preventgroupremoval[$group->id] = true;
+        }
 
         echo "<option value=\"{$group->id}\"$select title=\"$groupname\">$groupname</option>\n";
     }
@@ -231,7 +234,7 @@ $member_names = array();
 
 $atleastonemember = false;
 if ($singlegroup) {
-    if ($groupmemberroles = groups_get_members_by_role($groupids[0],$courseid,'u.id,u.firstname,u.lastname')) {
+    if ($groupmemberroles = groups_get_members_by_role($groupids[0], $courseid, 'u.id, ' . get_all_user_name_fields(true, 'u'))) {
         foreach($groupmemberroles as $roleid=>$roledata) {
             echo '<optgroup label="'.s($roledata->name).'">';
             foreach($roledata->users as $member) {
@@ -260,17 +263,16 @@ echo '</table>'."\n";
 echo '</div>'."\n";
 echo '</form>'."\n";
 
-if (ajaxenabled()) {
-    $PAGE->requires->js_init_call('M.core_group.init_index', array($CFG->wwwroot, $courseid));
-}
+$PAGE->requires->js_init_call('M.core_group.init_index', array($CFG->wwwroot, $courseid));
+$PAGE->requires->js_init_call('M.core_group.groupslist', array($preventgroupremoval));
 
 echo $OUTPUT->footer();
 
 /**
  * Returns the first button action with the given prefix, taken from
  * POST or GET, otherwise returns false.
- * See /lib/moodlelib.php function optional_param.
- * @param $prefix 'act_' as in 'action'.
+ * @see /lib/moodlelib.php function optional_param().
+ * @param string $prefix 'act_' as in 'action'.
  * @return string The action without the prefix, or false if no action found.
  */
 function groups_param_action($prefix = 'act_') {
