@@ -21,35 +21,13 @@
  *
  * Included by import.ph
  *
- * @package    mod
- * @subpackage lesson
+ * @package mod_lesson
  * @copyright  1999 onwards Martin Dougiamas  {@link http://moodle.com}
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  **/
 
 defined('MOODLE_INTERNAL') || die();
 
-/**#@+
- * The core question types.
- *
- * These used to be in lib/questionlib.php, but are being deprecated. Copying them
- * here to keep this code working for now.
- */
-if (!defined('SHORTANSWER')) {
-    define("SHORTANSWER",   "shortanswer");
-    define("TRUEFALSE",     "truefalse");
-    define("MULTICHOICE",   "multichoice");
-    define("RANDOM",        "random");
-    define("MATCH",         "match");
-    define("RANDOMSAMATCH", "randomsamatch");
-    define("DESCRIPTION",   "description");
-    define("NUMERICAL",     "numerical");
-    define("MULTIANSWER",   "multianswer");
-    define("CALCULATED",    "calculated");
-    define("ESSAY",         "essay");
-}
-/**#@-*/
-
 /**
  * Given some question info and some data about the the answers
  * this function parses, organises and saves the question
@@ -59,10 +37,10 @@ if (!defined('SHORTANSWER')) {
  * Lifted from mod/quiz/lib.php -
  *    1. all reference to oldanswers removed
  *    2. all reference to quiz_multichoice table removed
- *    3. In SHORTANSWER questions usecase is store in the qoption field
- *    4. In NUMERIC questions store the range as two answers
- *    5. TRUEFALSE options are ignored
- *    6. For MULTICHOICE questions with more than one answer the qoption field is true
+ *    3. In shortanswer questions usecase is store in the qoption field
+ *    4. In numeric questions store the range as two answers
+ *    5. truefalse options are ignored
+ *    6. For multichoice questions with more than one answer the qoption field is true
  *
  * @param opject $question Contains question data like question, type and answers.
  * @return object Returns $result->error or $result->notice.
@@ -116,7 +94,7 @@ function lesson_save_question_options($question, $lesson) {
             }
             break;
 
-        case LESSON_PAGE_NUMERICAL:   // Note similarities to SHORTANSWER
+        case LESSON_PAGE_NUMERICAL:   // Note similarities to shortanswer.
 
             $answers = array();
             $maxfraction = -1;
@@ -243,7 +221,7 @@ function lesson_save_question_options($question, $lesson) {
                 $totalfraction = round($totalfraction,2);
                 if ($totalfraction != 1) {
                     $totalfraction = $totalfraction * 100;
-                    $result->notice = get_string("fractionsaddwrong", "quiz", $totalfraction);
+                    $result->notice = get_string("fractionsaddwrong", "qtype_multichoice", $totalfraction);
                     return $result;
                 }
             }
@@ -262,6 +240,7 @@ function lesson_save_question_options($question, $lesson) {
             // The first answer should always be the correct answer
             $correctanswer = clone($defaultanswer);
             $correctanswer->answer = get_string('thatsthecorrectanswer', 'lesson');
+            $correctanswer->jumpto = LESSON_NEXTPAGE;
             $DB->insert_record("lesson_answers", $correctanswer);
 
             // The second answer should always be the wrong answer
@@ -303,13 +282,14 @@ function lesson_save_question_options($question, $lesson) {
 class qformat_default {
 
     var $displayerrors = true;
-    var $category = NULL;
+    var $category = null;
     var $questionids = array();
-    var $qtypeconvert = array(NUMERICAL   => LESSON_PAGE_NUMERICAL,
-                              MULTICHOICE => LESSON_PAGE_MULTICHOICE,
-                              TRUEFALSE   => LESSON_PAGE_TRUEFALSE,
-                              SHORTANSWER => LESSON_PAGE_SHORTANSWER,
-                              MATCH       => LESSON_PAGE_MATCHING
+    protected $importcontext = null;
+    var $qtypeconvert = array('numerical'   => LESSON_PAGE_NUMERICAL,
+                               'multichoice' => LESSON_PAGE_MULTICHOICE,
+                               'truefalse'   => LESSON_PAGE_TRUEFALSE,
+                               'shortanswer' => LESSON_PAGE_SHORTANSWER,
+                               'match'       => LESSON_PAGE_MATCHING
                               );
 
     // Importing functions
@@ -317,6 +297,44 @@ class qformat_default {
         return false;
     }
 
+    function set_importcontext($context) {
+        $this->importcontext = $context;
+    }
+
+    /**
+     * Handle parsing error
+     *
+     * @param string $message information about error
+     * @param string $text imported text that triggered the error
+     * @param string $questionname imported question name
+     */
+    protected function error($message, $text='', $questionname='') {
+        $importerrorquestion = get_string('importerrorquestion', 'question');
+
+        echo "<div class=\"importerror\">\n";
+        echo "<strong>$importerrorquestion $questionname</strong>";
+        if (!empty($text)) {
+            $text = s($text);
+            echo "<blockquote>$text</blockquote>\n";
+        }
+        echo "<strong>$message</strong>\n";
+        echo "</div>";
+    }
+
+    /**
+     * Import for questiontype plugins
+     * @param mixed $data The segment of data containing the question
+     * @param object $question processed (so far) by standard import code if appropriate
+     * @param object $extra mixed any additional format specific data that may be passed by the format
+     * @param string $qtypehint hint about a question type from format
+     * @return object question object suitable for save_options() or false if cannot handle
+     */
+    public function try_importing_using_qtypes($data, $question = null, $extra = null,
+            $qtypehint = '') {
+
+        return false;
+    }
+
     function importpreprocess() {
         // Does any pre-processing that may be desired
         return true;
@@ -343,6 +361,13 @@ class qformat_default {
                 $this->count_questions($questions)), 'notifysuccess');
 
         $count = 0;
+        $addquestionontop = false;
+        if ($pageid == 0) {
+            $addquestionontop = true;
+            $updatelessonpage = $DB->get_record('lesson_pages', array('lessonid' => $lesson->id, 'prevpageid' => 0));
+        } else {
+            $updatelessonpage = $DB->get_record('lesson_pages', array('lessonid' => $lesson->id, 'id' => $pageid));
+        }
 
         $unsupportedquestions = 0;
 
@@ -352,11 +377,11 @@ class qformat_default {
                 case 'category':
                     break;
                 // the good ones
-                case SHORTANSWER :
-                case NUMERICAL :
-                case TRUEFALSE :
-                case MULTICHOICE :
-                case MATCH :
+                case 'shortanswer' :
+                case 'numerical' :
+                case 'truefalse' :
+                case 'multichoice' :
+                case 'match' :
                     $count++;
 
                     //Show nice formated question in one line.
@@ -366,12 +391,12 @@ class qformat_default {
                     $newpage->lessonid = $lesson->id;
                     $newpage->qtype = $this->qtypeconvert[$question->qtype];
                     switch ($question->qtype) {
-                        case SHORTANSWER :
+                        case 'shortanswer' :
                             if (isset($question->usecase)) {
                                 $newpage->qoption = $question->usecase;
                             }
                             break;
-                        case MULTICHOICE :
+                        case 'multichoice' :
                             if (isset($question->single)) {
                                 $newpage->qoption = !$question->single;
                             }
@@ -386,14 +411,6 @@ class qformat_default {
                     $newpage->contents = $question->questiontext;
                     $newpage->contentsformat = isset($question->questionformat) ? $question->questionformat : FORMAT_HTML;
 
-                    // Sometimes, questiontext is not a simple text, but one array
-                    // containing both text and format, so we need to support here
-                    // that case with the following dirty patch. MDL-35147
-                    if (is_array($question->questiontext)) {
-                        $newpage->contents = isset($question->questiontext['text']) ? $question->questiontext['text'] : '';
-                        $newpage->contentsformat = isset($question->questiontext['format']) ? $question->questiontext['format'] : FORMAT_HTML;
-                    }
-
                     // set up page links
                     if ($pageid) {
                         // the new page follows on from this page
@@ -406,7 +423,6 @@ class qformat_default {
                         $newpageid = $DB->insert_record("lesson_pages", $newpage);
                         // update the linked list
                         $DB->set_field("lesson_pages", "nextpageid", $newpageid, array("id" => $pageid));
-
                     } else {
                         // new page is the first page
                         // get the existing (first) page (if any)
@@ -425,12 +441,22 @@ class qformat_default {
                             $DB->set_field("lesson_pages", "prevpageid", $newpageid, array("id" => $page->id));
                         }
                     }
+
                     // reset $pageid and put the page ID in $question, used in save_question_option()
                     $pageid = $newpageid;
                     $question->id = $newpageid;
 
                     $this->questionids[] = $question->id;
 
+                    // Import images in question text.
+                    if (isset($question->questiontextitemid)) {
+                        $questiontext = file_save_draft_area_files($question->questiontextitemid,
+                                $this->importcontext->id, 'mod_lesson', 'page_contents', $newpageid,
+                                null , $question->questiontext);
+                        // Update content with recoded urls.
+                        $DB->set_field("lesson_pages", "contents", $questiontext, array("id" => $newpageid));
+                    }
+
                     // Now to save all the answers and type-specific options
 
                     $question->lessonid = $lesson->id; // needed for foreign key
@@ -452,7 +478,14 @@ class qformat_default {
                     $unsupportedquestions++;
                     break;
             }
-
+        }
+        // Update the prev links if there were existing pages.
+        if (!empty($updatelessonpage)) {
+            if ($addquestionontop) {
+                $DB->set_field("lesson_pages", "prevpageid", $pageid, array("id" => $updatelessonpage->id));
+            } else {
+                $DB->set_field("lesson_pages", "prevpageid", $pageid, array("id" => $updatelessonpage->nextpageid));
+            }
         }
         if ($unsupportedquestions) {
             echo $OUTPUT->notification(get_string('unknownqtypesnotimported', 'lesson', $unsupportedquestions));
@@ -531,14 +564,15 @@ class qformat_default {
     }
 
 
-    function readquestion($lines) {
+    protected function readquestion($lines) {
     /// Given an array of lines known to define a question in
     /// this format, this function converts it into a question
     /// object suitable for processing and insertion into Moodle.
 
-        echo "<p>This flash question format has not yet been completed!</p>";
+        // We should never get there unless the qformat plugin is broken.
+        throw new coding_exception('Question format plugin is missing important code: readquestion.');
 
-        return NULL;
+        return null;
     }
 
     /**
@@ -565,7 +599,7 @@ class qformat_default {
         $name = clean_param($name, PARAM_TEXT); // Matches what the question editing form does.
         $name = trim($name);
         $trimlength = 251;
-        while (textlib::strlen($name) > 255 && $trimlength > 0) {
+        while (core_text::strlen($name) > 255 && $trimlength > 0) {
             $name = shorten_text($name, $trimlength);
             $trimlength -= 10;
         }
@@ -617,7 +651,12 @@ class qformat_default {
     protected function format_question_text($question) {
         $formatoptions = new stdClass();
         $formatoptions->noclean = true;
-        return html_to_text(format_text($question->questiontext,
+        // The html_to_text call strips out all URLs, but format_text complains
+        // if it finds @@PLUGINFILE@@ tokens. So, we need to replace
+        // @@PLUGINFILE@@ with a real URL, but it doesn't matter what.
+        // We use http://example.com/.
+        $text = str_replace('@@PLUGINFILE@@/', 'http://example.com/', $question->questiontext);
+        return html_to_text(format_text($text,
                 $question->questiontextformat, $formatoptions), 0, false);
     }
 
@@ -654,8 +693,8 @@ class qformat_based_on_xml extends qformat_default {
             "&#8212;" => "-",
         );
         $str = strtr($str, $html_code_list);
-        // Use textlib entities_to_utf8 function to convert only numerical entities.
-        $str = textlib::entities_to_utf8($str, false);
+        // Use core_text entities_to_utf8 function to convert only numerical entities.
+        $str = core_text::entities_to_utf8($str, false);
         return $str;
     }
 
