@@ -1,17 +1,35 @@
 <?php
-    require_once($CFG->dirroot.'/mod/scorm/locallib.php');
+// This file is part of Moodle - http://moodle.org/
+//
+// Moodle is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// Moodle is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
 
-    if (isset($userdata->status)) {
-        if (!isset($userdata->{'cmi.exit'}) || (($userdata->{'cmi.exit'} == 'time-out') || ($userdata->{'cmi.exit'} == 'normal'))) {
-                $userdata->entry = 'ab-initio';
+require_once($CFG->dirroot.'/mod/scorm/locallib.php');
+
+if (isset($userdata->status)) {
+    if (!isset($userdata->{'cmi.exit'}) || (($userdata->{'cmi.exit'} == 'time-out') || ($userdata->{'cmi.exit'} == 'normal'))) {
+            $userdata->entry = 'ab-initio';
+    } else {
+        if (isset($userdata->{'cmi.exit'}) && (($userdata->{'cmi.exit'} == 'suspend') || ($userdata->{'cmi.exit'} == 'logout'))) {
+            $userdata->entry = 'resume';
         } else {
-            if (isset($userdata->{'cmi.exit'}) && (($userdata->{'cmi.exit'} == 'suspend') || ($userdata->{'cmi.exit'} == 'logout'))) {
-                $userdata->entry = 'resume';
-            } else {
-                $userdata->entry = '';
-            }
+            $userdata->entry = '';
         }
     }
+}
+if (!isset($currentorg)) {
+    $currentorg = '';
+}
 ?>
 
 // Used need to debug cmi content (if you uncomment this, you must comment the definition inside SCORMapi1_3)
@@ -72,9 +90,9 @@ function SCORMapi1_3() {
     var CMIInteger = '^\\d+$';
     var CMISInteger = '^-?([0-9]+)$';
     var CMIDecimal = '^-?([0-9]{1,5})(\\.[0-9]{1,18})?$';
-    var CMIIdentifier = '^\\S{0,250}[a-zA-Z0-9]$';
+    var CMIIdentifier = '^\\S{1,250}[a-zA-Z0-9]$';
     var CMIShortIdentifier = '^[\\w\.]{1,250}$';
-    var CMILongIdentifier = '^\\S{0,4000}[a-zA-Z0-9]$';
+    var CMILongIdentifier = '^(?:(?!urn:)\\S{1,4000}|urn:[A-Za-z0-9-]{1,31}:\\S{1,4000})$';
     var CMIFeedback = '^.*$'; // This must be redefined
     var CMIIndex = '[._](\\d+).';
     var CMIIndexStore = '.N(\\d+).';
@@ -84,18 +102,18 @@ function SCORMapi1_3() {
     var CMIExit = '^time-out$|^suspend$|^logout$|^normal$|^$';
     var CMIType = '^true-false$|^choice$|^(long-)?fill-in$|^matching$|^performance$|^sequencing$|^likert$|^numeric$|^other$';
     var CMIResult = '^correct$|^incorrect$|^unanticipated$|^neutral$|^-?([0-9]{1,4})(\\.[0-9]{1,18})?$';
-    var NAVEvent = '^previous$|^continue$|^exit$|^exitAll$|^abandon$|^abandonAll$|^suspendAll$|^{target=\\S{0,200}[a-zA-Z0-9]}choice$';
+    var NAVEvent = '^previous$|^continue$|^exit$|^exitAll$|^abandon$|^abandonAll$|^suspendAll$|^\{target=\\S{0,200}[a-zA-Z0-9]\}choice|jump$';
     var NAVBoolean = '^unknown$|^true$|^false$';
     var NAVTarget = '^previous$|^continue$|^choice.{target=\\S{0,200}[a-zA-Z0-9]}$'
     // Children lists
-    var cmi_children = '_version, comments_from_learner, comments_from_lms, completion_status, credit, entry, exit, interactions, launch_data, learner_id, learner_name, learner_preference, location, max_time_allowed, mode, objectives, progress_measure, scaled_passing_score, score, session_time, success_status, suspend_data, time_limit_action, total_time';
-    var comments_children = 'comment, timestamp, location';
-    var score_children = 'max, raw, scaled, min';
-    var objectives_children = 'progress_measure, completion_status, success_status, description, score, id';
+    var cmi_children = '_version,comments_from_learner,comments_from_lms,completion_status,credit,entry,exit,interactions,launch_data,learner_id,learner_name,learner_preference,location,max_time_allowed,mode,objectives,progress_measure,scaled_passing_score,score,session_time,success_status,suspend_data,time_limit_action,total_time';
+    var comments_children = 'comment,timestamp,location';
+    var score_children = 'max,raw,scaled,min';
+    var objectives_children = 'progress_measure,completion_status,success_status,description,score,id';
     var correct_responses_children = 'pattern';
-    var student_data_children = 'mastery_score, max_time_allowed, time_limit_action';
-    var student_preference_children = 'audio_level, audio_captioning, delivery_speed, language';
-    var interactions_children = 'id, type, objectives, timestamp, correct_responses, weighting, learner_response, result, latency, description';
+    var student_data_children = 'mastery_score,max_time_allowed,time_limit_action';
+    var student_preference_children = 'audio_level,audio_captioning,delivery_speed,language';
+    var interactions_children = 'id,type,objectives,timestamp,correct_responses,weighting,learner_response,result,latency,description';
     // Data ranges
     var scaled_range = '-1#1';
     var audio_range = '0#*';
@@ -159,11 +177,11 @@ function SCORMapi1_3() {
         'cmi.comments_from_lms.n.comment':{'format':CMILangString4000, 'mod':'r'},
         'cmi.comments_from_lms.n.location':{'format':CMIString250, 'mod':'r'},
         'cmi.comments_from_lms.n.timestamp':{'format':CMITime, 'mod':'r'},
-        'cmi.completion_status':{'defaultvalue':'<?php echo isset($userdata->{'cmi.completion_status'})?$userdata->{'cmi.completion_status'}:'unknown' ?>', 'format':CMICStatus, 'mod':'rw'},
-        'cmi.completion_threshold':{'defaultvalue':<?php echo isset($userdata->threshold)?'\''.$userdata->threshold.'\'':'null' ?>, 'mod':'r'},
-        'cmi.credit':{'defaultvalue':'<?php echo isset($userdata->credit)?$userdata->credit:'' ?>', 'mod':'r'},
+        'cmi.completion_status':{'defaultvalue':'<?php echo !empty($userdata->{'cmi.completion_status'})?$userdata->{'cmi.completion_status'}:'unknown' ?>', 'format':CMICStatus, 'mod':'rw'},
+        'cmi.completion_threshold':{'defaultvalue':<?php echo !empty($userdata->threshold)?'\''.$userdata->threshold.'\'':'null' ?>, 'mod':'r'},
+        'cmi.credit':{'defaultvalue':'<?php echo !empty($userdata->credit)?$userdata->credit:'' ?>', 'mod':'r'},
         'cmi.entry':{'defaultvalue':'<?php echo $userdata->entry ?>', 'mod':'r'},
-        'cmi.exit':{'defaultvalue':'<?php echo isset($userdata->{'cmi.exit'})?$userdata->{'cmi.exit'}:'' ?>', 'format':CMIExit, 'mod':'w'},
+        'cmi.exit':{'defaultvalue':'<?php echo !empty($userdata->{'cmi.exit'})?$userdata->{'cmi.exit'}:'' ?>', 'format':CMIExit, 'mod':'w'},
         'cmi.interactions._children':{'defaultvalue':interactions_children, 'mod':'r'},
         'cmi.interactions._count':{'mod':'r', 'defaultvalue':'0'},
         'cmi.interactions.n.id':{'pattern':CMIIndex, 'format':CMILongIdentifier, 'mod':'rw'},
@@ -178,16 +196,16 @@ function SCORMapi1_3() {
         'cmi.interactions.n.result':{'pattern':CMIIndex, 'format':CMIResult, 'mod':'rw'},
         'cmi.interactions.n.latency':{'pattern':CMIIndex, 'format':CMITimespan, 'mod':'rw'},
         'cmi.interactions.n.description':{'pattern':CMIIndex, 'format':CMILangString250, 'mod':'rw'},
-        'cmi.launch_data':{'defaultvalue':<?php echo isset($userdata->datafromlms)?'\''.$userdata->datafromlms.'\'':'null' ?>, 'mod':'r'},
+        'cmi.launch_data':{'defaultvalue':<?php echo !empty($userdata->datafromlms)?'\''.$userdata->datafromlms.'\'':'null' ?>, 'mod':'r'},
         'cmi.learner_id':{'defaultvalue':'<?php echo $userdata->student_id ?>', 'mod':'r'},
         'cmi.learner_name':{'defaultvalue':'<?php echo $userdata->student_name ?>', 'mod':'r'},
         'cmi.learner_preference._children':{'defaultvalue':student_preference_children, 'mod':'r'},
-        'cmi.learner_preference.audio_level':{'defaultvalue':'1', 'format':CMIDecimal, 'range':audio_range, 'mod':'rw'},
-        'cmi.learner_preference.language':{'defaultvalue':'', 'format':CMILang, 'mod':'rw'},
-        'cmi.learner_preference.delivery_speed':{'defaultvalue':'1', 'format':CMIDecimal, 'range':speed_range, 'mod':'rw'},
-        'cmi.learner_preference.audio_captioning':{'defaultvalue':'0', 'format':CMISInteger, 'range':text_range, 'mod':'rw'},
-        'cmi.location':{'defaultvalue':<?php echo isset($userdata->{'cmi.location'})?'\''.$userdata->{'cmi.location'}.'\'':'null' ?>, 'format':CMIString1000, 'mod':'rw'},
-        'cmi.max_time_allowed':{'defaultvalue':<?php echo isset($userdata->maxtimeallowed)?'\''.$userdata->maxtimeallowed.'\'':'null' ?>, 'mod':'r'},
+        'cmi.learner_preference.audio_level':{'defaultvalue':<?php echo !empty($userdata->{'cmi.learner_preference.audio_level'})?'\''.$userdata->{'cmi.learner_preference.audio_level'}.'\'':'\'1\'' ?>, 'format':CMIDecimal, 'range':audio_range, 'mod':'rw'},
+        'cmi.learner_preference.language':{'defaultvalue':<?php echo !empty($userdata->{'cmi.learner_preference.language'})?'\''.$userdata->{'cmi.learner_preference.language'}.'\'':'\'\'' ?>, 'format':CMILang, 'mod':'rw'},
+        'cmi.learner_preference.delivery_speed':{'defaultvalue':<?php echo !empty($userdata->{'cmi.learner_preference.delivery_speed'})?'\''.$userdata->{'cmi.learner_preference.delivery_speed'}.'\'':'\'1\'' ?>, 'format':CMIDecimal, 'range':speed_range, 'mod':'rw'},
+        'cmi.learner_preference.audio_captioning':{'defaultvalue':<?php echo !empty($userdata->{'cmi.learner_preference.audio_captioning'})?'\''.$userdata->{'cmi.learner_preference.audio_captioning'}.'\'':'\'0\'' ?>, 'format':CMISInteger, 'range':text_range, 'mod':'rw'},
+        'cmi.location':{'defaultvalue':<?php echo !empty($userdata->{'cmi.location'})?'\''.$userdata->{'cmi.location'}.'\'':'null' ?>, 'format':CMIString1000, 'mod':'rw'},
+        'cmi.max_time_allowed':{'defaultvalue':<?php echo !empty($userdata->attemptAbsoluteDurationLimit)?'\''.$userdata->attemptAbsoluteDurationLimit.'\'':'null' ?>, 'mod':'r'},
         'cmi.mode':{'defaultvalue':'<?php echo $userdata->mode ?>', 'mod':'r'},
         'cmi.objectives._children':{'defaultvalue':objectives_children, 'mod':'r'},
         'cmi.objectives._count':{'mod':'r', 'defaultvalue':'0'},
@@ -201,18 +219,18 @@ function SCORMapi1_3() {
         'cmi.objectives.n.completion_status':{'defaultvalue':'unknown', 'pattern':CMIIndex, 'format':CMICStatus, 'mod':'rw'},
         'cmi.objectives.n.progress_measure':{'defaultvalue':null, 'format':CMIDecimal, 'range':progress_range, 'mod':'rw'},
         'cmi.objectives.n.description':{'pattern':CMIIndex, 'format':CMILangString250, 'mod':'rw'},
-        'cmi.progress_measure':{'defaultvalue':<?php echo isset($userdata->{'cmi.progess_measure'})?'\''.$userdata->{'cmi.progress_measure'}.'\'':'null' ?>, 'format':CMIDecimal, 'range':progress_range, 'mod':'rw'},
-        'cmi.scaled_passing_score':{'defaultvalue':<?php echo isset($userdata->{'cmi.scaled_passing_score'})?'\''.$userdata->{'cmi.scaled_passing_score'}.'\'':'null' ?>, 'format':CMIDecimal, 'range':scaled_range, 'mod':'r'},
+        'cmi.progress_measure':{'defaultvalue':<?php echo !empty($userdata->{'cmi.progress_measure'})?'\''.$userdata->{'cmi.progress_measure'}.'\'':'null' ?>, 'format':CMIDecimal, 'range':progress_range, 'mod':'rw'},
+        'cmi.scaled_passing_score':{'defaultvalue':<?php echo !empty($userdata->{'cmi.scaled_passing_score'})?'\''.$userdata->{'cmi.scaled_passing_score'}.'\'':'null' ?>, 'format':CMIDecimal, 'range':scaled_range, 'mod':'r'},
         'cmi.score._children':{'defaultvalue':score_children, 'mod':'r'},
-        'cmi.score.scaled':{'defaultvalue':<?php echo isset($userdata->{'cmi.score.scaled'})?'\''.$userdata->{'cmi.score.scaled'}.'\'':'null' ?>, 'format':CMIDecimal, 'range':scaled_range, 'mod':'rw'},
-        'cmi.score.raw':{'defaultvalue':<?php echo isset($userdata->{'cmi.score.raw'})?'\''.$userdata->{'cmi.score.raw'}.'\'':'null' ?>, 'format':CMIDecimal, 'mod':'rw'},
-        'cmi.score.min':{'defaultvalue':<?php echo isset($userdata->{'cmi.score.min'})?'\''.$userdata->{'cmi.score.min'}.'\'':'null' ?>, 'format':CMIDecimal, 'mod':'rw'},
-        'cmi.score.max':{'defaultvalue':<?php echo isset($userdata->{'cmi.score.max'})?'\''.$userdata->{'cmi.score.max'}.'\'':'null' ?>, 'format':CMIDecimal, 'mod':'rw'},
+        'cmi.score.scaled':{'defaultvalue':<?php echo !empty($userdata->{'cmi.score.scaled'})?'\''.$userdata->{'cmi.score.scaled'}.'\'':'null' ?>, 'format':CMIDecimal, 'range':scaled_range, 'mod':'rw'},
+        'cmi.score.raw':{'defaultvalue':<?php echo !empty($userdata->{'cmi.score.raw'})?'\''.$userdata->{'cmi.score.raw'}.'\'':'null' ?>, 'format':CMIDecimal, 'mod':'rw'},
+        'cmi.score.min':{'defaultvalue':<?php echo !empty($userdata->{'cmi.score.min'})?'\''.$userdata->{'cmi.score.min'}.'\'':'null' ?>, 'format':CMIDecimal, 'mod':'rw'},
+        'cmi.score.max':{'defaultvalue':<?php echo !empty($userdata->{'cmi.score.max'})?'\''.$userdata->{'cmi.score.max'}.'\'':'null' ?>, 'format':CMIDecimal, 'mod':'rw'},
         'cmi.session_time':{'format':CMITimespan, 'mod':'w', 'defaultvalue':'PT0H0M0S'},
-        'cmi.success_status':{'defaultvalue':'<?php echo isset($userdata->{'cmi.success_status'})?$userdata->{'cmi.success_status'}:'unknown' ?>', 'format':CMISStatus, 'mod':'rw'},
-        'cmi.suspend_data':{'defaultvalue':<?php echo isset($userdata->{'cmi.suspend_data'})?'\''.$userdata->{'cmi.suspend_data'}.'\'':'null' ?>, 'format':CMIString64000, 'mod':'rw'},
-        'cmi.time_limit_action':{'defaultvalue':<?php echo isset($userdata->timelimitaction)?'\''.$userdata->timelimitaction.'\'':'null' ?>, 'mod':'r'},
-        'cmi.total_time':{'defaultvalue':'<?php echo isset($userdata->{'cmi.total_time'})?$userdata->{'cmi.total_time'}:'PT0H0M0S' ?>', 'mod':'r'},
+        'cmi.success_status':{'defaultvalue':'<?php echo !empty($userdata->{'cmi.success_status'})?$userdata->{'cmi.success_status'}:'unknown' ?>', 'format':CMISStatus, 'mod':'rw'},
+        'cmi.suspend_data':{'defaultvalue':<?php echo !empty($userdata->{'cmi.suspend_data'})?'\''.$userdata->{'cmi.suspend_data'}.'\'':'null' ?>, 'format':CMIString64000, 'mod':'rw'},
+        'cmi.time_limit_action':{'defaultvalue':<?php echo !empty($userdata->timelimitaction)?'\''.$userdata->timelimitaction.'\'':'null' ?>, 'mod':'r'},
+        'cmi.total_time':{'defaultvalue':'<?php echo !empty($userdata->{'cmi.total_time'})?$userdata->{'cmi.total_time'}:'PT0H0M0S' ?>', 'mod':'r'},
         'adl.nav.request':{'defaultvalue':'_none_', 'format':NAVEvent, 'mod':'rw'}
     };
     //
@@ -263,6 +281,7 @@ function SCORMapi1_3() {
     var Initialized = false;
     var Terminated = false;
     var diagnostic = "";
+    var errorCode = "0";
 
     function Initialize (param) {
         errorCode = "0";
@@ -271,8 +290,7 @@ function SCORMapi1_3() {
                 Initialized = true;
                 errorCode = "0";
                 <?php
-                    if (debugging('',DEBUG_DEVELOPER)) {
-//                        echo 'alert("Initialized SCORM 1.3");';
+                    if (scorm_debugging($scorm)) {
                         echo 'LogAPICall("Initialize", param, "", errorCode);';
                     }
                 ?>
@@ -288,8 +306,7 @@ function SCORMapi1_3() {
             errorCode = "201";
         }
         <?php
-            if (debugging('',DEBUG_DEVELOPER)) {
-//                echo 'alert("Initialize: "+GetErrorString(errorCode));';
+            if (scorm_debugging($scorm)) {
                 echo 'LogAPICall("Initialize", param, "", errorCode);';
             }
         ?>
@@ -300,40 +317,57 @@ function SCORMapi1_3() {
         errorCode = "0";
         if (param == "") {
             if ((Initialized) && (!Terminated)) {
+                var AJAXResult = StoreData(cmi,true);
                 <?php
-                    if (debugging('',DEBUG_DEVELOPER)) {
-//                        echo 'alert("Terminated SCORM 1.3");';
-                        echo 'LogAPICall("Terminate", param, "", 0);';
+                    if (scorm_debugging($scorm)) {
+                        echo 'LogAPICall("Terminate", "AJAXResult", AJAXResult, 0);';
                     }
                 ?>
-                Initialized = false;
-                Terminated = true;
-                var result = StoreData(cmi,true);
-                if (adl.nav.request != '_none_') {
-                    switch (adl.nav.request) {
-                        case 'continue':
-                            setTimeout('top.nextSCO();',500);
-                        break;
-                        case 'previous':
-                            setTimeout('top.prevSCO();',500);
-                        break;
-                        case 'choice':
-                        break;
-                        case 'exit':
-                        break;
-                        case 'exitAll':
-                        break;
-                        case 'abandon':
-                        break;
-                        case 'abandonAll':
-                        break;
+                result = ('true' == AJAXResult) ? 'true' : 'false';
+                errorCode = ('true' == result)? '0' : '101'; // General exception for any AJAX fault
+                <?php
+                    if (scorm_debugging($scorm)) {
+                        echo 'LogAPICall("Terminate", "result", result, errorCode);';
                     }
+                ?>
+                if ('true' == result) {
+                    Initialized = false;
+                    Terminated = true;
+                    if (adl.nav.request != '_none_') {
+                        switch (adl.nav.request) {
+                            case 'continue':
+                                setTimeout('mod_scorm_launch_next_sco();',500);
+                            break;
+                            case 'previous':
+                                setTimeout('mod_scorm_launch_prev_sco();',500);
+                            break;
+                            case 'choice':
+                            break;
+                            case 'exit':
+                            break;
+                            case 'exitAll':
+                            break;
+                            case 'abandon':
+                            break;
+                            case 'abandonAll':
+                            break;
+                        }
+                    } else {
+                        if (<?php echo $scorm->auto ?> == 1) {
+                            setTimeout('mod_scorm_launch_next_sco();',500);
+                        }
+                    }
+                    // trigger TOC update
+                    var sURL = "<?php echo $CFG->wwwroot; ?>" + "/mod/scorm/prereqs.php?a=<?php echo $scorm->id ?>&scoid=<?php echo $scoid ?>&attempt=<?php echo $attempt ?>&mode=<?php echo $mode ?>&currentorg=<?php echo $currentorg ?>&sesskey=<?php echo sesskey(); ?>";
+                    var callback = M.mod_scorm.connectPrereqCallback;
+                    YUI().use('io-base', function(Y) {
+                        Y.on('io:complete', callback.success, Y);
+                        Y.io(sURL);
+                    });
                 } else {
-                    if (<?php echo $scorm->auto ?> == 1) {
-                        setTimeout('top.nextSCO();',500);
-                    }
+                    diagnostic = "Failure calling the Terminate remote callback: the server replied with HTTP Status " + AJAXResult;
                 }
-                return "true";
+                return result;
             } else {
                 if (Terminated) {
                     errorCode = "113";
@@ -345,8 +379,8 @@ function SCORMapi1_3() {
             errorCode = "201";
         }
         <?php
-            if (debugging('',DEBUG_DEVELOPER)) {
-                echo 'alert("Terminate: "+GetErrorString(errorCode));';
+            if (scorm_debugging($scorm)) {
+                echo 'LogAPICall("Terminate", param, "", errorCode);';
             }
         ?>
         return "false";
@@ -377,8 +411,7 @@ function SCORMapi1_3() {
                             if ((typeof eval(subelement) != "undefined") && (eval(subelement) != null)) {
                                 errorCode = "0";
                                 <?php
-                                    if (debugging('',DEBUG_DEVELOPER)) {
-//                                        echo 'alert("GetValue("+element+") -> "+eval(element));';
+                                    if (scorm_debugging($scorm)) {
                                         echo 'LogAPICall("GetValue", element, eval(element), 0);';
                                     }
                                 ?>
@@ -443,8 +476,7 @@ function SCORMapi1_3() {
             }
         }
         <?php
-            if (debugging('',DEBUG_DEVELOPER)) {
-//                echo 'alert("GetValue("+element+") -> "+GetErrorString(errorCode));';
+            if (scorm_debugging($scorm)) {
                 echo 'LogAPICall("GetValue", element, "", errorCode);';
             }
         ?>
@@ -578,9 +610,6 @@ function SCORMapi1_3() {
                                                             } else {
                                                                 nodes[0] = value;
                                                             }
-                                                            if (interactiontype == 'choice' && nodes.length == 1) {
-                                                              alert('not enough choices: ' + element);
-                                                            }
                                                             if ((nodes.length > 0) && (nodes.length <= correct_responses[interactiontype].max)) {
                                                                 errorCode = CRcheckValueNodes (element, interactiontype, nodes, value, errorCode);
                                                             } else if (nodes.length > correct_responses[interactiontype].max) {
@@ -695,8 +724,8 @@ function SCORMapi1_3() {
                                                 }
                                              break;
                                          case 'cmi.interactions.n.correct_responses.n.pattern':
-											 subel= subelement.split('.');
-											 subel1= 'cmi.interactions.'+subel[2];
+	                                         subel= subelement.split('.');
+                                             subel1= 'cmi.interactions.'+subel[2];
 
                                                 if (typeof eval(subel1+'.type') == "undefined") {
                                                     errorCode="408";
@@ -743,8 +772,7 @@ function SCORMapi1_3() {
                                             eval(element+'=value;');
                                             errorCode = "0";
                                             <?php
-                                                if (debugging('',DEBUG_DEVELOPER)) {
-//                                                    echo 'alert("SetValue("+element+","+value+") -> OK");';
+                                                if (scorm_debugging($scorm)) {
                                                     echo 'LogAPICall("SetValue", element, value, errorCode);';
                                                 }
                                             ?>
@@ -759,8 +787,7 @@ function SCORMapi1_3() {
                                     eval(element+'=value;');
                                     errorCode = "0";
                                     <?php
-                                        if (debugging('',DEBUG_DEVELOPER)) {
-//                                           echo 'alert("SetValue("+element+","+value+") -> OK");';
+                                        if (scorm_debugging($scorm)) {
                                             echo 'LogAPICall("SetValue", element, value, errorCode);';
                                         }
                                     ?>
@@ -787,7 +814,7 @@ function SCORMapi1_3() {
             }
         }
         <?php
-            if (debugging('',DEBUG_DEVELOPER)) {
+            if (scorm_debugging($scorm)) {
                 echo 'LogAPICall("SetValue", element, value, errorCode);';
             }
         ?>
@@ -912,14 +939,23 @@ function SCORMapi1_3() {
         errorCode = "0";
         if (param == "") {
             if ((Initialized) && (!Terminated)) {
-                result = StoreData(cmi,false);
+                var AJAXResult = StoreData(cmi,false);
                 <?php
-                    if (debugging('',DEBUG_DEVELOPER)) {
-                        echo 'LogAPICall("Commit", param, "", 0);';
-                        //echo 'alert("Data Commited");';
+                    if (scorm_debugging($scorm)) {
+                        echo 'LogAPICall("Commit", "AJAXResult", AJAXResult, 0);';
                     }
                 ?>
-                return "true";
+                var result = ('true' == AJAXResult) ? 'true' : 'false';
+                errorCode = ('true' == result)? '0' : '101'; // General exception for any AJAX fault
+                <?php
+                    if (scorm_debugging($scorm)) {
+                        echo 'LogAPICall("Commit", "result", result, errorCode);';
+                    }
+                ?>
+                if ('false' == result) {
+                    diagnostic = "Failure calling the Commit remote callback: the server replied with HTTP Status " + AJAXResult;
+                }
+                return result;
             } else {
                 if (Terminated) {
                     errorCode = "143";
@@ -931,17 +967,16 @@ function SCORMapi1_3() {
             errorCode = "201";
         }
         <?php
-            if (debugging('',DEBUG_DEVELOPER)) {
-                echo 'LogAPICall("Commit", param, "", 0);';
-//                echo 'alert("Commit: "+GetErrorString(errorCode));';
+            if (scorm_debugging($scorm)) {
+                echo 'LogAPICall("Commit", param, "", errorCode);';
             }
         ?>
         return "false";
     }
 
     function GetLastError () {
-     <?php
-        if (debugging('',DEBUG_DEVELOPER)) {
+    <?php
+        if (scorm_debugging($scorm)) {
             echo 'LogAPICall("GetLastError", "", "", errorCode);';
         }
     ?>
@@ -1032,14 +1067,14 @@ function SCORMapi1_3() {
                 break;
             }
             <?php
-            if (debugging('',DEBUG_DEVELOPER)) {
+            if (scorm_debugging($scorm)) {
                 echo 'LogAPICall("GetErrorString", param,  errorString, 0);';
             }
              ?>
             return errorString;
         } else {
            <?php
-            if (debugging('',DEBUG_DEVELOPER)) {
+            if (scorm_debugging($scorm)) {
                 echo 'LogAPICall("GetErrorString", param,  "No error string found!", 0);';
             }
              ?>
@@ -1050,14 +1085,14 @@ function SCORMapi1_3() {
     function GetDiagnostic (param) {
         if (diagnostic != "") {
             <?php
-                if (debugging('',DEBUG_DEVELOPER)) {
+                if (scorm_debugging($scorm)) {
                     echo 'LogAPICall("GetDiagnostic", param, diagnostic, 0);';
                 }
             ?>
             return diagnostic;
         }
         <?php
-            if (debugging('',DEBUG_DEVELOPER)) {
+            if (scorm_debugging($scorm)) {
                 echo 'LogAPICall("GetDiagnostic", param, param, 0);';
             }
         ?>
@@ -1101,7 +1136,7 @@ function SCORMapi1_3() {
 
     function AddTime (first, second) {
         <?php
-//            if (debugging('',DEBUG_DEVELOPER)) {
+//            if (scorm_debugging($scorm)) {
 //                echo 'alert("AddTime: "+first+" + "+second);';
 //            }
         ?>
@@ -1231,18 +1266,8 @@ function SCORMapi1_3() {
         datastring += navrequest;
         datastring += '&attempt=<?php echo $attempt ?>';
         datastring += '&scoid=<?php echo $scoid ?>';
-        <?php
-//            if (debugging('',DEBUG_DEVELOPER)) {
-//                echo 'popupwin(datastring);';
-//            }
-        ?>
         var myRequest = NewHttpReq();
-        var result = DoRequest(myRequest,"<?php p($CFG->wwwroot) ?>/mod/scorm/datamodel.php","id=<?php p($id) ?>&sesskey=<?php p($USER->sesskey) ?>"+datastring);
-        <?php
-//            if (debugging('',DEBUG_DEVELOPER)) {
-//                echo 'popupwin(result);';
-//            }
-        ?>
+        var result = DoRequest(myRequest,"<?php p($CFG->wwwroot) ?>/mod/scorm/datamodel.php","id=<?php p($id) ?>&a=<?php p($a) ?>&sesskey=<?php echo sesskey() ?>"+datastring);
         var results = String(result).split('\n');
         if ((results.length > 2) && (navrequest != '')) {
             eval(results[2]);
@@ -1266,8 +1291,7 @@ var API_1484_11 = new SCORMapi1_3();
 
 <?php
 // pull in the debugging utilities
-if (debugging('',DEBUG_DEVELOPER)) {
+if (scorm_debugging($scorm)) {
     include_once($CFG->dirroot.'/mod/scorm/datamodels/debug.js.php');
     echo 'AppendToLog("Moodle SCORM 1.3 API Loaded, Activity: '.$scorm->name.', SCO: '.$sco->identifier.'", 0);';
 }
- ?>
\ No newline at end of file
