@@ -44,30 +44,30 @@ class qtype_multianswer extends question_type {
     public function get_question_options($question) {
         global $DB, $OUTPUT;
 
-        // Get relevant data indexed by positionkey from the multianswers table
+        // Get relevant data indexed by positionkey from the multianswers table.
         $sequence = $DB->get_field('question_multianswer', 'sequence',
                 array('question' => $question->id), '*', MUST_EXIST);
 
         $wrappedquestions = $DB->get_records_list('question', 'id',
                 explode(',', $sequence), 'id ASC');
 
-        // We want an array with question ids as index and the positions as values
+        // We want an array with question ids as index and the positions as values.
         $sequence = array_flip(explode(',', $sequence));
         array_walk($sequence, create_function('&$val', '$val++;'));
 
         // If a question is lost, the corresponding index is null
         // so this null convention is used to test $question->options->questions
         // before using the values.
-        // first all possible questions from sequence are nulled
-        // then filled with the data if available in  $wrappedquestions
+        // First all possible questions from sequence are nulled
+        // then filled with the data if available in  $wrappedquestions.
         foreach ($sequence as $seq) {
             $question->options->questions[$seq] = '';
         }
 
         foreach ($wrappedquestions as $wrapped) {
             question_bank::get_qtype($wrapped->qtype)->get_question_options($wrapped);
-            // for wrapped questions the maxgrade is always equal to the defaultmark,
-            // there is no entry in the question_instances table for them
+            // For wrapped questions the maxgrade is always equal to the defaultmark,
+            // there is no entry in the question_instances table for them.
             $wrapped->maxmark = $wrapped->defaultmark;
             $question->options->questions[$sequence[$wrapped->id]] = $wrapped;
         }
@@ -84,12 +84,12 @@ class qtype_multianswer extends question_type {
 
         // This function needs to be able to handle the case where the existing set of wrapped
         // questions does not match the new set of wrapped questions so that some need to be
-        // created, some modified and some deleted
+        // created, some modified and some deleted.
         // Unfortunately the code currently simply overwrites existing ones in sequence. This
         // will make re-marking after a re-ordering of wrapped questions impossible and
         // will also create difficulties if questiontype specific tables reference the id.
 
-        // First we get all the existing wrapped questions
+        // First we get all the existing wrapped questions.
         if (!$oldwrappedids = $DB->get_field('question_multianswer', 'sequence',
                 array('question' => $question->id))) {
             $oldwrappedquestions = array();
@@ -101,7 +101,7 @@ class qtype_multianswer extends question_type {
         $sequence = array();
         foreach ($question->options->questions as $wrapped) {
             if (!empty($wrapped)) {
-                // if we still have some old wrapped question ids, reuse the next of them
+                // If we still have some old wrapped question ids, reuse the next of them.
 
                 if (is_array($oldwrappedquestions) &&
                         $oldwrappedquestion = array_shift($oldwrappedquestions)) {
@@ -109,12 +109,12 @@ class qtype_multianswer extends question_type {
                     if ($oldwrappedquestion->qtype != $wrapped->qtype) {
                         switch ($oldwrappedquestion->qtype) {
                             case 'multichoice':
-                                $DB->delete_records('question_multichoice',
-                                        array('question' => $oldwrappedquestion->id));
+                                $DB->delete_records('qtype_multichoice_options',
+                                        array('questionid' => $oldwrappedquestion->id));
                                 break;
                             case 'shortanswer':
-                                $DB->delete_records('question_shortanswer',
-                                        array('question' => $oldwrappedquestion->id));
+                                $DB->delete_records('qtype_shortanswer_options',
+                                        array('questionid' => $oldwrappedquestion->id));
                                 break;
                             case 'numerical':
                                 $DB->delete_records('question_numerical',
@@ -133,19 +133,19 @@ class qtype_multianswer extends question_type {
             $wrapped->name = $question->name;
             $wrapped->parent = $question->id;
             $previousid = $wrapped->id;
-            // save_question strips this extra bit off the category again.
+            // Save_question strips this extra bit off the category again.
             $wrapped->category = $question->category . ',1';
             $wrapped = question_bank::get_qtype($wrapped->qtype)->save_question(
                     $wrapped, clone($wrapped));
             $sequence[] = $wrapped->id;
             if ($previousid != 0 && $previousid != $wrapped->id) {
-                // for some reasons a new question has been created
-                // so delete the old one
+                // For some reasons a new question has been created
+                // so delete the old one.
                 question_delete_question($previousid);
             }
         }
 
-        // Delete redundant wrapped questions
+        // Delete redundant wrapped questions.
         if (is_array($oldwrappedquestions) && count($oldwrappedquestions)) {
             foreach ($oldwrappedquestions as $oldwrappedquestion) {
                 question_delete_question($oldwrappedquestion->id);
@@ -165,7 +165,7 @@ class qtype_multianswer extends question_type {
             }
         }
 
-        $this->save_hints($question);
+        $this->save_hints($question, true);
     }
 
     public function save_question($authorizedquestion, $form) {
@@ -183,6 +183,10 @@ class qtype_multianswer extends question_type {
         return parent::save_question($question, $form);
     }
 
+    protected function make_hint($hint) {
+        return question_hint_with_parts::load_from_record($hint);
+    }
+
     public function delete_question($questionid, $contextid) {
         global $DB;
         $DB->delete_records('question_multianswer', array('question' => $questionid));
@@ -190,7 +194,7 @@ class qtype_multianswer extends question_type {
         parent::delete_question($questionid, $contextid);
     }
 
-    protected function initialise_question_instance($question, $questiondata) {
+    protected function initialise_question_instance(question_definition $question, $questiondata) {
         parent::initialise_question_instance($question, $questiondata);
 
         $bits = preg_split('/\{#(\d+)\}/', $question->questiontext,
@@ -236,10 +240,10 @@ class qtype_multianswer extends question_type {
 }
 
 
-// ANSWER_ALTERNATIVE regexes
+// ANSWER_ALTERNATIVE regexes.
 define('ANSWER_ALTERNATIVE_FRACTION_REGEX',
        '=|%(-?[0-9]+)%');
-// for the syntax '(?<!' see http://www.perl.com/doc/manual/html/pod/perlre.html#item_C
+// For the syntax '(?<!' see http://www.perl.com/doc/manual/html/pod/perlre.html#item_C.
 define('ANSWER_ALTERNATIVE_ANSWER_REGEX',
         '.+?(?<!\\\\|&|&amp;)(?=[~#}]|$)');
 define('ANSWER_ALTERNATIVE_FEEDBACK_REGEX',
@@ -249,24 +253,24 @@ define('ANSWER_ALTERNATIVE_REGEX',
        '(' . ANSWER_ALTERNATIVE_ANSWER_REGEX . ')' .
        '(#(' . ANSWER_ALTERNATIVE_FEEDBACK_REGEX .'))?');
 
-// Parenthesis positions for ANSWER_ALTERNATIVE_REGEX
+// Parenthesis positions for ANSWER_ALTERNATIVE_REGEX.
 define('ANSWER_ALTERNATIVE_REGEX_PERCENTILE_FRACTION', 2);
 define('ANSWER_ALTERNATIVE_REGEX_FRACTION', 1);
 define('ANSWER_ALTERNATIVE_REGEX_ANSWER', 3);
 define('ANSWER_ALTERNATIVE_REGEX_FEEDBACK', 5);
 
 // NUMBER_FORMATED_ALTERNATIVE_ANSWER_REGEX is used
-// for identifying numerical answers in ANSWER_ALTERNATIVE_REGEX_ANSWER
+// for identifying numerical answers in ANSWER_ALTERNATIVE_REGEX_ANSWER.
 define('NUMBER_REGEX',
         '-?(([0-9]+[.,]?[0-9]*|[.,][0-9]+)([eE][-+]?[0-9]+)?)');
 define('NUMERICAL_ALTERNATIVE_REGEX',
         '^(' . NUMBER_REGEX . ')(:' . NUMBER_REGEX . ')?$');
 
-// Parenthesis positions for NUMERICAL_FORMATED_ALTERNATIVE_ANSWER_REGEX
+// Parenthesis positions for NUMERICAL_FORMATED_ALTERNATIVE_ANSWER_REGEX.
 define('NUMERICAL_CORRECT_ANSWER', 1);
 define('NUMERICAL_ABS_ERROR_MARGIN', 6);
 
-// Remaining ANSWER regexes
+// Remaining ANSWER regexes.
 define('ANSWER_TYPE_DEF_REGEX',
         '(NUMERICAL|NM)|(MULTICHOICE|MC)|(MULTICHOICE_V|MCV)|(MULTICHOICE_H|MCH)|' .
                 '(SHORTANSWER|SA|MW)|(SHORTANSWER_C|SAC|MWC)');
@@ -280,7 +284,7 @@ define('ANSWER_REGEX',
         . ANSWER_ALTERNATIVE_REGEX
         . ')*)\}');
 
-// Parenthesis positions for singulars in ANSWER_REGEX
+// Parenthesis positions for singulars in ANSWER_REGEX.
 define('ANSWER_REGEX_NORM', 1);
 define('ANSWER_REGEX_ANSWER_TYPE_NUMERICAL', 3);
 define('ANSWER_REGEX_ANSWER_TYPE_MULTICHOICE', 4);
@@ -291,7 +295,7 @@ define('ANSWER_REGEX_ANSWER_TYPE_SHORTANSWER_C', 8);
 define('ANSWER_REGEX_ALTERNATIVES', 9);
 
 function qtype_multianswer_extract_question($text) {
-    // $text is an array [text][format][itemid]
+    // Variable $text is an array [text][format][itemid].
     $question = new stdClass();
     $question->qtype = 'multianswer';
     $question->questiontext = $text;
@@ -299,8 +303,9 @@ function qtype_multianswer_extract_question($text) {
     $question->generalfeedback['format'] = FORMAT_HTML;
     $question->generalfeedback['itemid'] = '';
 
+    $question->options = new stdClass();
     $question->options->questions = array();
-    $question->defaultmark = 0; // Will be increased for each answer norm
+    $question->defaultmark = 0; // Will be increased for each answer norm.
 
     for ($positionkey = 1;
             preg_match('/'.ANSWER_REGEX.'/s', $question->questiontext['text'], $answerregs);
@@ -379,7 +384,7 @@ function qtype_multianswer_extract_question($text) {
 
         // Each $wrapped simulates a $form that can be processed by the
         // respective save_question and save_question_options methods of the
-        // wrapped questiontypes
+        // wrapped questiontypes.
         $wrapped->answer   = array();
         $wrapped->fraction = array();
         $wrapped->feedback = array();
@@ -420,7 +425,7 @@ function qtype_multianswer_extract_question($text) {
                 } else {
                     $wrapped->tolerance["$answerindex"] = 0;
                 }
-            } else { // Tolerance can stay undefined for non numerical questions
+            } else { // Tolerance can stay undefined for non numerical questions.
                 // Undo quoting done by the HTML editor.
                 $answer = html_entity_decode(
                         $altregs[ANSWER_ALTERNATIVE_REGEX_ANSWER], ENT_QUOTES, 'UTF-8');
