@@ -1,5 +1,4 @@
 <?php
-
 // This file is part of Moodle - http://moodle.org/
 //
 // Moodle is free software: you can redistribute it and/or modify
@@ -18,58 +17,76 @@
 /**
  * Display profile for a particular user
  *
+ * @package core_user
  * @copyright 1999 Martin Dougiamas  http://dougiamas.com
  * @license http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
- * @package user
  */
 
 require_once("../config.php");
 require_once($CFG->dirroot.'/user/profile/lib.php');
 require_once($CFG->dirroot.'/tag/lib.php');
 require_once($CFG->libdir . '/filelib.php');
+require_once($CFG->libdir . '/badgeslib.php');
 
-$id        = optional_param('id', 0, PARAM_INT);   // user id
-$courseid  = optional_param('course', SITEID, PARAM_INT);   // course id (defaults to Site)
+$id             = optional_param('id', 0, PARAM_INT); // User id.
+$courseid       = optional_param('course', SITEID, PARAM_INT); // course id (defaults to Site).
+$showallcourses = optional_param('showallcourses', 0, PARAM_INT);
 
-if (empty($id)) {            // See your own profile by default
+// See your own profile by default.
+if (empty($id)) {
     require_login();
     $id = $USER->id;
 }
 
-if ($courseid == SITEID) {   // Since Moodle 2.0 all site-level profiles are shown by profile.php
-    redirect($CFG->wwwroot.'/user/profile.php?id='.$id);  // Immediate redirect
+if ($courseid == SITEID) {   // Since Moodle 2.0 all site-level profiles are shown by profile.php.
+    redirect($CFG->wwwroot.'/user/profile.php?id='.$id);  // Immediate redirect.
 }
 
-$PAGE->set_url('/user/view.php', array('id'=>$id,'course'=>$courseid));
+$PAGE->set_url('/user/view.php', array('id' => $id, 'course' => $courseid));
 
-$user = $DB->get_record('user', array('id'=>$id), '*', MUST_EXIST);
-$course = $DB->get_record('course', array('id'=>$courseid), '*', MUST_EXIST);
+$user = $DB->get_record('user', array('id' => $id), '*', MUST_EXIST);
+$course = $DB->get_record('course', array('id' => $courseid), '*', MUST_EXIST);
 $currentuser = ($user->id == $USER->id);
 
-$systemcontext = get_context_instance(CONTEXT_SYSTEM);
-$coursecontext = get_context_instance(CONTEXT_COURSE, $course->id);
-$usercontext   = get_context_instance(CONTEXT_USER, $user->id, IGNORE_MISSING);
+$systemcontext = context_system::instance();
+$coursecontext = context_course::instance($course->id);
+$usercontext   = context_user::instance($user->id, IGNORE_MISSING);
 
-// Require login first
+// Check we are not trying to view guest's profile.
 if (isguestuser($user)) {
-    // can not view profile of guest - thre is nothing to see there
+    // Can not view profile of guest - thre is nothing to see there.
     print_error('invaliduserid');
 }
 
+$PAGE->set_context($coursecontext);
+
 if (!empty($CFG->forceloginforprofiles)) {
-    require_login(); // we can not log in to course due to the parent hack below
+    require_login(); // We can not log in to course due to the parent hack below.
+
+    // Guests do not have permissions to view anyone's profile if forceloginforprofiles is set.
+    if (isguestuser()) {
+        echo $OUTPUT->header();
+        echo $OUTPUT->confirm(get_string('guestcantaccessprofiles', 'error'),
+                              get_login_url(),
+                              $CFG->wwwroot);
+        echo $OUTPUT->footer();
+        die;
+    }
 }
 
-$PAGE->set_context($coursecontext);
 $PAGE->set_course($course);
-$PAGE->set_pagetype('course-view-' . $course->format);  // To get the blocks exactly like the course
-$PAGE->add_body_class('path-user');                     // So we can style it independently
+$PAGE->set_pagetype('course-view-' . $course->format);  // To get the blocks exactly like the course.
+$PAGE->add_body_class('path-user');                     // So we can style it independently.
 $PAGE->set_other_editing_capability('moodle/course:manageactivities');
 
+// Set the Moodle docs path explicitly because the default behaviour
+// of inhereting the pagetype will lead to an incorrect docs location.
+$PAGE->set_docs_path('user/profile');
+
 $isparent = false;
 
 if (!$currentuser and !$user->deleted
-  and $DB->record_exists('role_assignments', array('userid'=>$USER->id, 'contextid'=>$usercontext->id))
+  and $DB->record_exists('role_assignments', array('userid' => $USER->id, 'contextid' => $usercontext->id))
   and has_capability('moodle/user:viewdetails', $usercontext)) {
     // TODO: very ugly hack - do not force "parents" to enrol into course their child is enrolled in,
     //       this way they may access the profile where they get overview of grades and child activity in course,
@@ -78,9 +95,9 @@ if (!$currentuser and !$user->deleted
     $isparent = true;
     $PAGE->navigation->set_userid_for_parent_checks($id);
 } else {
-    // normal course
+    // Normal course.
     require_login($course);
-    // what to do with users temporary accessing this course? should they see the details?
+    // What to do with users temporary accessing this course? should they see the details?
 }
 
 $strpersonalprofile = get_string('personalprofile');
@@ -89,10 +106,10 @@ $struser = get_string("user");
 
 $fullname = fullname($user, has_capability('moodle/site:viewfullnames', $coursecontext));
 
-/// Now test the actual capabilities and enrolment in course
+// Now test the actual capabilities and enrolment in course.
 if ($currentuser) {
-    // me
-    if (!is_viewing($coursecontext) && !is_enrolled($coursecontext)) { // Need to have full access to a course to see the rest of own info
+    if (!is_viewing($coursecontext) && !is_enrolled($coursecontext)) {
+        // Need to have full access to a course to see the rest of own info.
         echo $OUTPUT->header();
         echo $OUTPUT->heading(get_string('notenrolled', '', $fullname));
         if (!empty($_SERVER['HTTP_REFERER'])) {
@@ -103,20 +120,20 @@ if ($currentuser) {
     }
 
 } else {
-    // somebody else
+    // Somebody else.
     $PAGE->set_title("$strpersonalprofile: ");
     $PAGE->set_heading("$strpersonalprofile: ");
 
-    // check course level capabilities
-    if (!has_capability('moodle/user:viewdetails', $coursecontext) && // normal enrolled user or mnager
-        ($user->deleted or !has_capability('moodle/user:viewdetails', $usercontext))) {   // usually parent
+    // Check course level capabilities.
+    if (!has_capability('moodle/user:viewdetails', $coursecontext) && // Normal enrolled user or mnager.
+        ($user->deleted or !has_capability('moodle/user:viewdetails', $usercontext))) {   // Usually parent.
         print_error('cannotviewprofile');
     }
 
     if (!is_enrolled($coursecontext, $user->id)) {
         // TODO: the only potential problem is that managers and inspectors might post in forum, but the link
         //       to profile would not work - maybe a new capability - moodle/user:freely_acessile_profile_for_anybody
-        //       or test for course:inspect capability
+        //       or test for course:inspect capability.
         if (has_capability('moodle/role:assign', $coursecontext)) {
             $PAGE->navbar->add($fullname);
             echo $OUTPUT->header();
@@ -133,11 +150,15 @@ if ($currentuser) {
         exit;
     }
 
-    // If groups are in use and enforced throughout the course, then make sure we can meet in at least one course level group
-    if (groups_get_course_groupmode($course) == SEPARATEGROUPS and $course->groupmodeforce
-      and !has_capability('moodle/site:accessallgroups', $coursecontext) and !has_capability('moodle/site:accessallgroups', $coursecontext, $user->id)) {
+    // If groups are in use and enforced throughout the course, then make sure we can meet in at least one course level group.
+    // Except when we are a parent, in which case we would not be in any group.
+    if (groups_get_course_groupmode($course) == SEPARATEGROUPS
+            and $course->groupmodeforce
+            and !has_capability('moodle/site:accessallgroups', $coursecontext)
+            and !has_capability('moodle/site:accessallgroups', $coursecontext, $user->id)
+            and !$isparent) {
         if (!isloggedin() or isguestuser()) {
-            // do not use require_login() here because we might have already used require_login($course)
+            // Do not use require_login() here because we might have already used require_login($course).
             redirect(get_login_url());
         }
         $mygroups = array_keys(groups_get_all_groups($course->id, $USER->id, $course->defaultgroupingid, 'g.id, g.name'));
@@ -180,11 +201,23 @@ if ($user->deleted) {
     }
 }
 
-/// OK, security out the way, now we are showing the user
+// OK, security out the way, now we are showing the user.
+// Trigger a user profile viewed event.
+$event = \core\event\user_profile_viewed::create(array(
+    'objectid' => $user->id,
+    'relateduserid' => $user->id,
+    'courseid' => $course->id,
+    'context' => $coursecontext,
+    'other' => array(
+        'courseid' => $course->id,
+        'courseshortname' => $course->shortname,
+        'coursefullname' => $course->fullname
+    )
+));
+$event->add_record_snapshot('user', $user);
+$event->trigger();
 
-add_to_log($course->id, "user", "view", "view.php?id=$user->id&course=$course->id", "$user->id");
-
-/// Get the hidden field list
+// Get the hidden field list.
 if (has_capability('moodle/user:viewhiddendetails', $coursecontext)) {
     $hiddenfields = array();
 } else {
@@ -207,62 +240,68 @@ if (is_mnet_remote_user($user)) {
 }
 
 echo '<div class="userprofilebox clearfix"><div class="profilepicture">';
-echo $OUTPUT->user_picture($user, array('size'=>100));
+echo $OUTPUT->user_picture($user, array('size' => 100));
 echo '</div>';
 
-// Print the description
+// Print the description.
 echo '<div class="descriptionbox"><div class="description">';
 if ($user->description && !isset($hiddenfields['description'])) {
-    if (!empty($CFG->profilesforenrolledusersonly) && !$DB->record_exists('role_assignments', array('userid'=>$id))) {
+    if (!empty($CFG->profilesforenrolledusersonly) && !$DB->record_exists('role_assignments', array('userid' => $id))) {
         echo get_string('profilenotshown', 'moodle');
     } else {
         if ($courseid == SITEID) {
             $user->description = file_rewrite_pluginfile_urls($user->description, 'pluginfile.php', $usercontext->id, 'user', 'profile', null);
         } else {
-            // we have to make a little detour thought the course context to verify the access control for course profile
+            // We have to make a little detour thought the course context to verify the access control for course profile.
             $user->description = file_rewrite_pluginfile_urls($user->description, 'pluginfile.php', $coursecontext->id, 'user', 'profile', $user->id);
         }
-        $options = array('overflowdiv'=>true);
+        $options = array('overflowdiv' => true);
         echo format_text($user->description, $user->descriptionformat, $options);
     }
 }
 echo '</div>';
 
 
-// Print all the little details in a list
+// Print all the little details in a list.
 
-echo '<table class="list" summary="">';
-
-//checks were performed above that ensure that if we've got to here either the user
-//is viewing their own profile ($USER->id == $user->id) or $user is enrolled in the course
+echo html_writer::start_tag('dl', array('class' => 'list'));
+// Show email if any of the following conditions match.
+// 1. User is viewing his own profile.
+// 2. Has allowed everyone to see email
+// 3. User has allowed course members to can see email and current user is in same course
+// 4. Has either course:viewhiddenuserfields or site:viewuseridentity capability.
 if ($currentuser
-   or $user->maildisplay == 1 //allow everyone to see email address
-   or ($user->maildisplay == 2 && is_enrolled($coursecontext, $USER)) //fellow course members can see email. Already know $user is enrolled
-   or has_capability('moodle/course:useremail', $coursecontext)) {
-    print_row(get_string("email").":", obfuscate_mailto($user->email, ''));
+   or $user->maildisplay == 1
+   or ($user->maildisplay == 2 && is_enrolled($coursecontext, $USER))
+   or has_capability('moodle/course:viewhiddenuserfields', $coursecontext)
+   or has_capability('moodle/site:viewuseridentity', $coursecontext)) {
+    echo html_writer::tag('dt', get_string('email'));
+    echo html_writer::tag('dd', obfuscate_mailto($user->email, ''));
 }
 
-// Show last time this user accessed this course
+// Show last time this user accessed this course.
 if (!isset($hiddenfields['lastaccess'])) {
-    if ($lastaccess = $DB->get_record('user_lastaccess', array('userid'=>$user->id, 'courseid'=>$course->id))) {
+    if ($lastaccess = $DB->get_record('user_lastaccess', array('userid' => $user->id, 'courseid' => $course->id))) {
         $datestring = userdate($lastaccess->timeaccess)."&nbsp; (".format_time(time() - $lastaccess->timeaccess).")";
     } else {
         $datestring = get_string("never");
     }
-    print_row(get_string("lastaccess").":", $datestring);
+    echo html_writer::tag('dt', get_string('lastcourseaccess'));
+    echo html_writer::tag('dd', $datestring);
 }
 
-// Show roles in this course
+// Show roles in this course.
 if ($rolestring = get_user_roles_in_course($id, $course->id)) {
-    print_row(get_string('roles').':', $rolestring);
+    echo html_writer::tag('dt', get_string('roles'));
+    echo html_writer::tag('dd', $rolestring);
 }
 
-// Show groups this user is in
+// Show groups this user is in.
 if (!isset($hiddenfields['groups'])) {
     $accessallgroups = has_capability('moodle/site:accessallgroups', $coursecontext);
     if ($usergroups = groups_get_all_groups($course->id, $user->id)) {
         $groupstr = '';
-        foreach ($usergroups as $group){
+        foreach ($usergroups as $group) {
             if ($course->groupmode == SEPARATEGROUPS and !$accessallgroups and $user->id != $USER->id) {
                 if (!groups_is_member($group->id, $user->id)) {
                     continue;
@@ -272,89 +311,107 @@ if (!isset($hiddenfields['groups'])) {
             if ($course->groupmode != NOGROUPS) {
                 $groupstr .= ' <a href="'.$CFG->wwwroot.'/user/index.php?id='.$course->id.'&amp;group='.$group->id.'">'.format_string($group->name).'</a>,';
             } else {
-                $groupstr .= ' '.format_string($group->name); // the user/index.php shows groups only when course in group mode
+                $groupstr .= ' '.format_string($group->name); // The user/index.php shows groups only when course in group mode.
             }
         }
         if ($groupstr !== '') {
-            print_row(get_string("group").":", rtrim($groupstr, ', '));
+            echo html_writer::tag('dt', get_string('group'));
+            echo html_writer::tag('dd', rtrim($groupstr, ', '));
         }
     }
 }
 
-// Show other courses they may be in
+// Show other courses they may be in.
 if (!isset($hiddenfields['mycourses'])) {
-    if ($mycourses = enrol_get_all_users_courses($user->id, true, NULL, 'visible DESC,sortorder ASC')) {
+    if ($mycourses = enrol_get_all_users_courses($user->id, true, null, 'visible DESC,sortorder ASC')) {
         $shown = 0;
         $courselisting = '';
         foreach ($mycourses as $mycourse) {
             if ($mycourse->category) {
-                $ccontext = get_context_instance(CONTEXT_COURSE, $mycourse->id);;
-                $cfullname = format_string($mycourse->fullname, true, array('context' => $ccontext));
-                if ($mycourse->id != $course->id){
-                    $class = '';
+                context_helper::preload_from_record($mycourse);
+                $ccontext = context_course::instance($mycourse->id);
+                $cfullname = $ccontext->get_context_name(false);
+                if ($mycourse->id != $course->id) {
+                    $linkattributes = null;
                     if ($mycourse->visible == 0) {
                         if (!has_capability('moodle/course:viewhiddencourses', $ccontext)) {
                             continue;
                         }
-                        $class = 'class="dimmed"';
+                        $linkattributes['class'] = 'dimmed';
                     }
-                    $courselisting .= "<a href=\"{$CFG->wwwroot}/user/view.php?id={$user->id}&amp;course={$mycourse->id}\" $class >"
-                        . $cfullname . "</a>, ";
+                    $params = array('id' => $user->id, 'course' => $mycourse->id);
+                    if ($showallcourses) {
+                        $params['showallcourses'] = 1;
+                    }
+                    $url = new moodle_url('/user/view.php', $params);
+                    $courselisting .= html_writer::link($url, $ccontext->get_context_name(false), $linkattributes);
+                    $courselisting .= ', ';
                 } else {
                     $courselisting .= $cfullname . ", ";
                     $PAGE->navbar->add($cfullname);
                 }
             }
             $shown++;
-            if ($shown >= 20) {
-                $courselisting .= "...";
+            if (!$showallcourses && $shown >= 20) {
+                $url = new moodle_url('/user/view.php', array('id' => $user->id, 'course' => $courseid, 'showallcourses' => 1));
+                $courselisting .= html_writer::link($url, '...', array('title' => get_string('viewmore')));
                 break;
             }
         }
-        print_row(get_string('courseprofiles').':', rtrim($courselisting,', '));
+        echo html_writer::tag('dt', get_string('courseprofiles'));
+        echo html_writer::tag('dd', rtrim($courselisting, ', '));
     }
 }
 
+if (!empty($CFG->enablebadges) && !empty($CFG->badges_allowcoursebadges)) {
+    profile_display_badges($user->id, $courseid);
+}
+
 if (!isset($hiddenfields['suspended'])) {
     if ($user->suspended) {
-        print_row('', get_string('suspended', 'auth'));
+        echo html_writer::tag('dt', "&nbsp;");
+        echo html_writer::tag('dd', get_string('suspended', 'auth'));
     }
 }
 
-echo "</table></div></div>";
-
-// Print messaging link if allowed
+if (has_capability('moodle/user:viewlastip', $usercontext) && !isset($hiddenfields['lastip'])) {
+    if ($user->lastip) {
+        $iplookupurl = new moodle_url('/iplookup/index.php', array('ip' => $user->lastip, 'user' => $USER->id));
+        $ipstring = html_writer::link($iplookupurl, $user->lastip);
+    } else {
+        $ipstring = get_string("none");
+    }
+    echo html_writer::tag('dt', get_string('lastip'));
+    echo html_writer::tag('dd', $ipstring);
+}
+echo html_writer::end_tag('dl');
+echo "</div></div>"; // Closing desriptionbox and userprofilebox.
+// Print messaging link if allowed.
 if (isloggedin() && has_capability('moodle/site:sendmessage', $usercontext)
     && !empty($CFG->messaging) && !isguestuser() && !isguestuser($user) && ($USER->id != $user->id)) {
     echo '<div class="messagebox">';
-    echo '<a href="'.$CFG->wwwroot.'/message/index.php?id='.$user->id.'">'.get_string('messageselectadd').'</a>';
+    $sendmessageurl = new moodle_url('/message/index.php', array('id' => $user->id));
+    if ($courseid) {
+        $sendmessageurl->param('viewing', MESSAGE_VIEW_COURSE. $courseid);
+    }
+    echo html_writer::link($sendmessageurl, get_string('messageselectadd'));
     echo '</div>';
 }
 
-if ($currentuser || has_capability('moodle/user:viewdetails', $usercontext) || has_coursecontact_role($id)) {
+if (empty($CFG->forceloginforprofiles) || $currentuser || has_capability('moodle/user:viewdetails', $usercontext)
+        || has_coursecontact_role($id)) {
     echo '<div class="fullprofilelink">';
     echo html_writer::link($CFG->wwwroot.'/user/profile.php?id='.$id, get_string('fullprofile'));
     echo '</div>';
 }
 
-/// TODO Add more useful overview info for teachers here, see below
-
-/// Show links to notes made about this student (must click to display, for privacy)
-
-/// Recent comments made in this course
-
-/// Recent blogs associated with this course and items in it
+// TODO Add more useful overview info for teachers here, see below.
+// Show links to notes made about this student (must click to display, for privacy).
+// Recent comments made in this course.
+// Recent blogs associated with this course and items in it.
 
 
 
-echo '</div>';  // userprofile class
+echo '</div>';  // Userprofile class.
 
 echo $OUTPUT->footer();
-
-/// Functions ///////
-
-function print_row($left, $right) {
-    echo "\n<tr><th class=\"label c0\">$left</th><td class=\"info c1\">$right</td></tr>\n";
-}
-
-
